//<html><pre>
// KEHOME/java/getdata.java
// Sep/1/2003

// usage:
//	java getdata kbdata
//	subject predicate object
//	...
//	CONTROL-Z
// example:
//	java getdata http://tap.stanford.edu/data/
//	Person type ?
//	? subClassOf Person
//	CONTROL-Z

import java.io.*;
import java.util.regex.*;

import edu.stanford.TAP.Abbrev;
import edu.stanford.TAP.Client;
import edu.stanford.TAP.Resource;
import edu.stanford.TAP.XML;

/**
*	read TAP KB location from command line
*	create TAP KB client
*	read sentence(s) from standard input
*	parse to subject predicate object
*	get answer(s) from TAP KB
*/
public class getdata
{
    public static void main(String argv[])
    throws IOException
    {
	String info      = "# INFO: getdata: ";
	String error     = "# ERROR: getdata: ";
	String debug     = "no";

	String Subflag   = "includeSubClasses=yes";
	String Superflag = "includeSuperClasses=yes";
	String flags     = "";

	//String kbdata = System.getProperty("KBDATA");
	String kbdata = argv[0];
	Client tap = new Client(kbdata);

	BufferedReader in = new BufferedReader(
		new InputStreamReader(System.in));
	String line = "";
for (
	int linenumber = 0;
	line != null;
	++linenumber
) {
	System.err.print("gd$ ");
	try {
		line = in.readLine();
	} catch(IOException e) {
		return;
	}
	if (line == null) return;
	String[] sentence = Pattern.compile(" ").split(line);
	String subject   = sentence[0];
	String predicate = sentence[1];
	String object    = sentence[2];

	if (debug == "yes") {
	  System.err.println(info + "kbdata = "    + kbdata);
	  System.err.println(info + "subject = "   + subject);
	  System.err.println(info + "predicate = " + predicate);
	  System.err.println(info + "object = "    + object);
	}
	if (subject.equals("?")) {
		if (debug == "yes") System.err.println("# qtype = qsubject");
		flags = "inverse=yes";
		subject = object;
	} else if (predicate.equals("?")) {
		if (debug == "yes") System.err.println("# qtype = qverb");
		predicate = "arcs";
		flags = "inverse=no";
	} else if (object.equals("?")) {
		if (debug == "yes") System.err.println("# qtype = qobject");
		flags = "inverse=no";
	} else {
		if (debug == "yes") System.err.println("# qtype = unknown");
		System.err.println(error +
			"subject <"   + subject   + "> " +
			"predicate <" + predicate + "> " +
			"object <"    + object    + "> ");

		System.err.println("# usage: subject predicate ?");
		System.err.println("# usage: subject ? ?");
		System.err.println("# usage: ? predicate object");
		System.err.println("# usage: ? ? object");
		//return;
	}
	if (debug == "yes") {
	  System.err.println(info + "flags = "     + flags);
	}

	Resource answer = get_data(tap,subject,predicate,flags);
	if (answer.count() == 0) {
		System.err.println("  " + info + "count = " + answer.count());
	} else if (predicate.equals("arcs")) {
		if (flags.equals("inverse=no")) {
			get_property_values(tap,subject,answer);
		} else {
			print_column(answer);
		}
	} else {
		print_column(answer);
	}
} // end for
	return;
    } // end main()

/**
*	print one answer per line
*/
    public static void print_column(Resource answer)
    {
	for(int x = 0; x < answer.count(); ++x) {
	    System.out.println("  " + answer.item(x).value);
	}
	return;
    }
/**
*	print one line with comma separators
*/
    public static void print_line(Resource answer)
    {
	int n = answer.count();
	for(int x = 0; x < n-1; ++x) {
	System.out.print(answer.item(x).value + ", ");
	}
	System.out.println(answer.item(n-1).value);
	return;
    }

/**
*	get all property values of subject
*/
    public static void get_property_values(
	Client tap,
	String subject,
	Resource proplist
    )
    {
	String info  = "  # INFO: get_property_values: ";
	String debug = "no";
	String flags = "inverse=no";

	for(int x = 0; x < proplist.count(); ++x) {
	    String property = proplist.item(x).value;
	    if (debug == "yes") {
		System.out.println(info + "property = " + property);
	    }
	    if (property.equals("arcs")) continue;

	    Resource answer = get_data(tap,subject,property,flags);
	    System.out.print("  " + property + " = ");
	    print_line(answer);
	}
	return;
    } // end get_property_values()


/**
*	translate Qname to URIs
#	get data from TAP KB
*/
    public static Resource get_data(
	Client tap,
	String subject,
	String predicate,
	String flags
    )
    {
	String tapuri = "http://tap.stanford.edu/data/";
	String info   = "  # INFO: get_data: ";
	String debug  = "no";

	if (debug == "yes") {
	  System.out.println(info + "subject = "   + subject);
	  System.out.println(info + "predicate = " + predicate);
	  System.out.println(info + "flags = "     + flags);
	}

	String uriSubject = Qname.GetNameURI(subject);
	String uriPredicate = Qname.GetNameURI(predicate);

	if (debug == "yes") {
	  System.out.println(info + "tapuri = "       + tapuri);
	  System.out.println(info + "uriSubject = "   + uriSubject);
	  System.out.println(info + "uriPredicate = " + uriPredicate);
	}
	Resource answer = tap.GetData(uriSubject,uriPredicate,flags);
	for(int x = 0; x < answer.count(); ++x) {
	        String tapAnswer = answer.item(x).value;
	        String mkrAnswer = XML.SplitNamespace(tapAnswer).tag;
		answer.item(x).value = mkrAnswer;
	}
	return answer;
    } // end get_data()

} // end class

//</pre></html>
